using System;
using UnityEngine;

namespace Mod
{
    class EnergyCannonBehaviour : MonoBehaviour
    {
        TrailRenderer trail;
        float speed = 200f;
        float frequency = 150f;
        float magnitude = 25f;

        float damage = 350f;
        float ImpactStrength = 20f;

        bool closing;
        Vector3 pos;
        LayerMask layers;
        public bool createHitDecal = true;
        public bool SpawnExplosionEffect = true;

        void Awake()
        {
            trail = gameObject.AddComponent<TrailRenderer>();
            trail.time = 0.1f;
            trail.startWidth = 0.1f;
            trail.material = ModAPI.FindMaterial("VeryBright").ShallowClone();
            trail.endColor = new Color(255f, 194f, 0f, 0.1f);
            trail.startColor = new Color(255f, 194f, 0f, 0.1f);
            trail.numCapVertices = 2;

            layers = LayerMask.GetMask("Objects", "Bounds", "Debris");

            
            gameObject.SetLayer(9);
        }

        void Update()
        {
            if (closing)
            {
                trail.autodestruct = true;
                return;
            }
            pos = transform.position;
            var dir = transform.right;
            var distance = speed * Time.deltaTime;

            var ray = Utils.MaterialAwareRaycast(pos, dir, distance, layers, 16);

            if (ray.hit.HasValue)
            {
                var hit = ray.hit.Value;
                var hitPhys = ray.physicalBehaviour;

                if (hitPhys && hitPhys.ReflectsLasers)
                {
                    hitPhys.rigidbody.AddForceAtPosition(hit.normal * -ImpactStrength, hit.point, ForceMode2D.Impulse);
                    var newDir = Vector2.Reflect(transform.right, hit.normal);
                    transform.position = hit.point;
                    transform.right = newDir;
                    return;
                }

                if (hitPhys && hitPhys.AbsorbsLasers)
                {
                    transform.position = hit.point;
                    hit.collider.attachedRigidbody.AddForceAtPosition(hit.normal * -ImpactStrength, hit.point);
                    hitPhys.BurnProgress += 0.2f * UnityEngine.Random.value;
                    Impact(hit);
                    return;
                }
            }
            else
            {
                pos += dir * Time.deltaTime * speed;

                transform.position = pos + transform.right * Time.deltaTime;
            }
        }

        private void Impact(RaycastHit2D hit)
        {
            Vector2 normal = hit.normal;
            hit.transform.SendMessage("Shot", new Shot(normal, hit.point, this.damage, true), SendMessageOptions.DontRequireReceiver);
            this.closing = true;

            float dir = Mathf.Atan2(normal.y, normal.x);
            bool underWater = WaterBehaviour.IsPointUnderWater(hit.point);
            if (underWater)
            {
                ExplosionCreator.CreateUnderwaterExplosionEffect(new ExplosionCreator.ExplosionParameters
                {
                    Position = hit.point
                }, false);
            }
            else
            {
                Instantiate(ModAPI.FindSpawnable("Blaster").Prefab.GetComponent<BlasterBehaviour>().Bolt.GetComponent<BlasterboltBehaviour>().ImpactEffect, hit.point, Quaternion.Euler(0, 0, dir)).transform.up = normal;
                if (createHitDecal)
                    Instantiate(ModAPI.FindSpawnable("Blaster").Prefab.GetComponent<BlasterBehaviour>().Bolt.GetComponent<BlasterboltBehaviour>().HitDecal, hit.point, Quaternion.identity, hit.transform);
            }

            var physicalBehaviour = hit.collider.GetComponent<PhysicalBehaviour>();

            if (Global.main.PhysicalObjectsInWorldByTransform.TryGetValue(hit.transform, out physicalBehaviour) && physicalBehaviour.SimulateTemperature)
            {
                physicalBehaviour.Temperature = Mathf.Lerp(physicalBehaviour.Temperature, 500f, 0.05f / physicalBehaviour.ObjectArea);
            }

            ExplosionCreator.CreatePulseExplosion(hit.point, ImpactStrength, 1f, !underWater && SpawnExplosionEffect, false);
        }
    }
}