using UnityEngine;
using System;

namespace Mod
{
    class ModBlasterBehaviour : CanShoot
    {
        public float Recoil = 0.75f;
        public float InaccuracyMultiplier = 0.25f;
        public float Delay;
        public string[] particles;
        public Type ProjectileType;

        public AudioSource audio;
        public AudioClip[] clips;

        public Vector2 barrelPosition = new Vector2(0f, 0f);
        public Vector2 barrelDirection = Vector2.right;

        float t = 0f;

        void Awake()
        {
            audio = gameObject.AddComponent<AudioSource>();
            audio.outputAudioMixerGroup = Global.main.SoundEffects;
            audio.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
        }

        void Use()
        {
            Shoot();
        }

        void UseContinuous()
        {
            if(t > Delay)
            {
                Shoot();
            }
        }

        void Update()
        {
            t += 10f * Time.deltaTime;
        }

        public override void Shoot()
        {
            audio.pitch = UnityEngine.Random.Range(0.75f, 1.25f);
            audio.PlayOneShot(clips[UnityEngine.Random.Range(0, clips.Length - 1)]);

            t = 0f;
            GetComponent<PhysicalBehaviour>().rigidbody.AddForceAtPosition(BarrelDirection * -this.Recoil, BarrelPosition, ForceMode2D.Impulse);
            
            var bolt = new GameObject("bolt", ProjectileType);
            bolt.transform.position = BarrelPosition;
            bolt.transform.right = BarrelDirection + InaccuracyMultiplier * UnityEngine.Random.insideUnitCircle;
            for (int i = 0; i < particles.Length; i++)
            {
                ModAPI.CreateParticleEffect(particles[i], BarrelPosition);
            }
        }

        public override Vector2 BarrelPosition { get { return transform.TransformPoint(barrelPosition); } }
        public Vector2 BarrelDirection { get { return transform.TransformDirection(barrelDirection) * transform.localScale.x; } }

    }
}